/* FoundationAdditions.m created by lindberg on Mon 20-Dec-1999 */
/*-
 * Copyright (c) 2001 Doug McClure
 *
 * Original code from eogenerator Copyright (c) 2001 Carl Lindberg and Mike Gentry
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#import "FoundationAdditions.h"
#import <Foundation/Foundation.h>
#import <Foundation/NSPathUtilities.h>
#import <Foundation/NSDate.h>
#import <Foundation/NSScanner.h>
#import "BFEncodingUtilities.h"

@interface NSObject (KeyValueDeclarations)
- (id)valueForKeyPath:(NSString *)key;
@end

@interface NSObject (WarningAvoidance)
- (NSString *)name;  // method on EOAttribute/EOEntity etc.
@end


@implementation NSObject (ClassTestMethods)
- (BOOL)isString {
    return [self isKindOfClass:[NSString class]];
}

- (BOOL)isData {
    return [self isKindOfClass:[NSData class]];
}

- (BOOL)isInteger {
    if ( [self isKindOfClass:[NSString class]] ) {
        NSScanner *scanner = [NSScanner scannerWithString:(NSString *)self];
        double doubleValue;
        int intValue;

        if ( [scanner scanDouble:&doubleValue] && [scanner isAtEnd] ) {
            [scanner setScanLocation:0];

            if ( [scanner scanInt:&intValue] && [scanner isAtEnd]  ) {
                return (doubleValue == (double)intValue);
            }
        }
    }
    
    return NO;
}

- (BOOL)isReal {
    if ( [self isKindOfClass:[NSString class]] ) {
        NSScanner *scanner = [NSScanner scannerWithString:(NSString *)self];
        double doubleValue;
        int intValue;

        if ( [scanner scanDouble:&doubleValue] && [scanner isAtEnd] ) {
            [scanner setScanLocation:0];

            if ( [scanner scanInt:&intValue] && [scanner isAtEnd] ) {
                return (doubleValue != (double)intValue);
            }
        }
    }

    return NO;
}

- (BOOL)isArray {
    return [self isKindOfClass:[NSArray class]];
}

- (BOOL)isDictionary {
    return [self isKindOfClass:[NSDictionary class]];
}
@end


@implementation NSArray (EOGenerator)

- (NSArray *)reversedArray
{
    return [[self reverseObjectEnumerator] allObjects];
}

// does an @reversedArray KeyValue key
- (id)computeReversedArrayForKey:(NSString *) key
{
    if ([key length] == 0)
        return [self reversedArray];
    else
        return [[self reversedArray] valueForKeyPath:key];
}

static int sortByName(id obj1, id obj2, void *context)
{
    return [(NSString *)[obj1 name] compare:[obj2 name]];
}

// does an @sortedNameArray KeyValue key
- (id)computeSortedNameArrayForKey:(NSString *)key
{
    if ([key length] == 0)
        return [self sortedArrayUsingFunction:sortByName context:NULL];
    else
        return [[self sortedArrayUsingFunction:sortByName context:NULL] valueForKeyPath:key];
}

// does an @sortedStringArray KeyValue key
- (id)computeSortedStringArrayForKey:(NSString *)key
{
    if ([key length] == 0)
        return [self sortedArrayUsingSelector:@selector(compare:)];
    else
        return [[self sortedArrayUsingSelector:@selector(compare:)] valueForKeyPath:key];
}

@end

@implementation NSString (EOGenerator)

- (NSString *)initialCapitalString
{
    NSRange  firstLetterRange;
    NSString *firstLetterString;
    NSString *restOfString;

    if ([self length] == 0) return self;

    firstLetterRange  = [self rangeOfComposedCharacterSequenceAtIndex:0];
    firstLetterString = [[self substringWithRange:firstLetterRange] uppercaseString];
    restOfString      = [self substringFromIndex:NSMaxRange(firstLetterRange)];

    return [firstLetterString stringByAppendingString:restOfString];
}

/*"
 * Calls +#stringWithCStringNoCopy:length:freeWhenDone: with a value of YES for
 * freeWhenDone.
"*/
+ (id)stringWithCStringNoCopy:(char *)bytes length:(unsigned)length
{
    return [self stringWithCStringNoCopy:bytes length:length freeWhenDone:YES];
}

/*"
 * Returns an autoreleased string. Calls
 * -#initWithCStringNoCopy:length:freeWhenDone:.
"*/
+ (id)stringWithCStringNoCopy:(char *)bytes length:(unsigned)length freeWhenDone:(BOOL)freeBuffer
{
    return [[[self alloc] initWithCStringNoCopy:bytes length:length freeWhenDone:freeBuffer] autorelease];
}

@end


@implementation NSData (Base64Encoding)

/*"
 * Returns the Base-64 encoding string representing the receiver. Base-64
 * encoding (defined in RFC 1521) turns three bytes of binary data into four
 * bytes of ASCII data, and is used in MIME mail and other places. The resulting
 * string will not contain any newline characters; for more control use the
 * BFBase64EncodeData() function in BFEncodingUtilities.h.
"*/
- (NSString *)base64String
{
    return BFBase64EncodeData(self, nil);
}

@end


@implementation NSFileManager (EOGenerator)

- (BOOL)directoryExistsAtPath:(NSString *)path
{
    BOOL isDir = NO;
    BOOL isFile = [self fileExistsAtPath:path isDirectory:&isDir];
    return (isFile && isDir);
}

- (BOOL)regularFileExistsAtPath:(NSString *)path
{
    BOOL isDir = NO;
    BOOL isFile = [self fileExistsAtPath:path isDirectory:&isDir];
    return (isFile && !isDir);
}

- (NSString *)findFile:(NSString *)filename inSearchPath:(NSArray *)paths
{
    int i, count = [paths count];

    for (i=0; i<count; i++)
    {
        NSString *currPath = [paths objectAtIndex:i];
        NSString *fullPath = [currPath stringByAppendingPathComponent:filename];

        if ([self regularFileExistsAtPath:fullPath])
            return fullPath;
    }

    return nil;
}

- (void)touchPath:(NSString *)filePath
{
    NSMutableDictionary *attributes;

    attributes = [[NSMutableDictionary alloc] initWithCapacity:1];
    [attributes setObject:[NSDate date] forKey:NSFileModificationDate];
    [self changeFileAttributes:attributes atPath:filePath];
    [attributes release];
}

@end
